function resolveCharacterCodePoint(parameterName, item) {
    const codePoint = item.codePointAt(0);
    if (item.length > 0 && typeof codePoint !== "undefined" && item.length === String.fromCodePoint(codePoint).length) {
        return codePoint;
    }
    throw new RangeError(`\`${item}\` (parameter \`${parameterName}\`) is not a character which is in code point range 0 ~ 1114111!`);
}
function* rangeLooperNumerics(start, end, options) {
    const { endExclusive, startExclusive, step } = options;
    if (start <= end) {
        // Increment
        //@ts-ignore Overload.
        for (let current = start; current <= end; current += step) {
            if (!((startExclusive && current === start) ||
                (endExclusive && current === end))) {
                yield current;
            }
        }
    }
    else {
        // Decrement
        //@ts-ignore Overload.
        for (let current = start; current >= end; current -= step) {
            if (!((startExclusive && current === start) ||
                (endExclusive && current === end))) {
                yield current;
            }
        }
    }
}
function* rangeLooperCharacters(start, end, options) {
    for (const element of rangeLooperNumerics(start, end, options)) {
        yield String.fromCodePoint(element);
    }
}
export function rangeIterator(start, end, param2) {
    const options = (typeof param2 === "bigint" ||
        typeof param2 === "number") ? { step: param2 } : (param2 ?? {});
    const { startExclusive = false } = options;
    const endExclusive = options.endExclusive ?? options.exclusiveEnd ?? false;
    if (typeof start === "bigint" && typeof end === "bigint") {
        if (typeof options.step !== "undefined") {
            if (!(typeof options.step === "bigint" && options.step > 0n)) {
                throw new RangeError(`\`${options.step}\` (parameter \`options.step\`) is not a bigint which is > 0!`);
            }
        }
        return rangeLooperNumerics(start, end, {
            endExclusive,
            startExclusive,
            step: options.step ?? 1n
        });
    }
    if (typeof start === "number" && typeof end === "number") {
        if (typeof options.step !== "undefined") {
            if (!(typeof options.step === "number" && options.step > 0)) {
                throw new RangeError(`\`${options.step}\` (parameter \`options.step\`) is not a number which is > 0!`);
            }
        }
        return rangeLooperNumerics(start, end, {
            endExclusive,
            startExclusive,
            step: options.step ?? 1
        });
    }
    if (typeof start === "string" && typeof end === "string") {
        const startCodePoint = resolveCharacterCodePoint("start", start);
        const endCodePoint = resolveCharacterCodePoint("end", end);
        if (typeof options.step !== "undefined") {
            if (!(typeof options.step === "number" && Number.isSafeInteger(options.step) && options.step > 0)) {
                throw new RangeError(`\`${options.step}\` (parameter \`options.step\`) is not a number which is integer, safe, and > 0!`);
            }
        }
        return rangeLooperCharacters(startCodePoint, endCodePoint, {
            endExclusive,
            startExclusive,
            step: options.step ?? 1
        });
    }
    throw new TypeError(`Parameters \`start\` and \`end\` are not bigints, numbers, or strings (character)!`);
}
export default rangeIterator;
